@@ -7,6 +7,16 @@
 
 from __future__ import with_statement
 
+
+def mapForms():
+    return "Normal,Compact"
+
+def mapCompact(format, keyType, valueType):
+    if format == 2:
+        return True # Compact.
+    return isSimpleType(keyType) and isSimpleType(valueType)
+
+
 def qdump__QAtomicInt(d, value):
     d.putValue(value["_q_value"])
     d.putNumChild(0)
@@ -49,7 +59,7 @@ def qdump__QChar(d, value):
     d.putNumChild(0)
 
 
-def qform__QModelIndex():
+def qform__QAbstractItemModel():
     return "Normal,Enhanced"
 
 def qdump__QAbstractItemModel(d, value):
@@ -203,8 +213,14 @@ def qdump__QDateTime(d, value):
 
 
 def qdump__QDir(d, value):
-    d.putStringValue(value["d_ptr"]["d"].dereference()["path"])
     d.putNumChild(1)
+    data = value["d_ptr"]["d"].dereference()
+    try:
+        # Up to Qt 4.7
+        d.putStringValue(data["path"])
+    except:
+        # Qt 4.8 and later.
+        d.putStringValue(data["dirEntry"]["m_filePath"])
     if d.isExpanded():
         with Children(d):
             d.putCallItem("absolutePath", value, "absolutePath")
@@ -222,12 +238,14 @@ def qdump__QFile(d, value):
     d.putNumChild(1)
     if d.isExpanded():
         with Children(d):
+            base = value.type.fields()[0].type
+            d.putSubItem("[%s]" % str(base), value.cast(base), False)
             d.putCallItem("exists", value, "exists")
 
 
 def qdump__QFileInfo(d, value):
     try:
-        d.putStringValue(value["d_ptr"]["d"].dereference()["fileName"])
+        d.putStringValue(value["d_ptr"]["d"].dereference()["fileNames"][3])
     except:
         d.putPlainChildren(value)
         return
@@ -337,6 +355,9 @@ def qdump__QFlags(d, value):
     d.putNumChild(0)
 
 
+def qform__QHash():
+    return mapForms()
+
 def qdump__QHash(d, value):
 
     def hashDataFirstNode(value):
@@ -382,22 +403,19 @@ def qdump__QHash(d, value):
     d.putItemCount(size)
     d.putNumChild(size)
     if d.isExpanded():
-        isSimpleKey = isSimpleType(keyType)
-        isSimpleValue = isSimpleType(valueType)
+        isCompact = mapCompact(d.currentItemFormat(), keyType, valueType)
         node = hashDataFirstNode(value)
         innerType = e_ptr.dereference().type
         childType = innerType
-        if isSimpleKey and isSimpleValue:
+        if isCompact:
             childType = valueType
         with Children(d, size, maxNumChild=1000, childType=childType):
             for i in d.childRange():
                 it = node.dereference().cast(innerType)
                 with SubItem(d, i):
-                    key = it["key"]
-                    val = it["value"]
-                    if isSimpleKey and isSimpleValue:
-                        d.putName(key)
-                        d.putItem(val)
+                    if isCompact:
+                        d.putMapName(it["key"])
+                        d.putItem(it["value"])
                         d.putType(valueType)
                     else:
                         d.putItem(it)
@@ -410,11 +428,11 @@ def qdump__QHashNode(d, value):
     key = value["key"]
     val = value["value"]
 
-    if isSimpleType(keyType) and isSimpleType(valueType):
-        d.putName(key)
-        d.putValue(val)
-    else:
-        d.putValue(" ")
+    #if isSimpleType(keyType) and isSimpleType(valueType):
+    #    d.putName(key)
+    #    d.putValue(val)
+    #else:
+    d.putValue(" ")
 
     d.putNumChild(2)
     if d.isExpanded():
@@ -501,7 +519,7 @@ def qdump__QImage(d, value):
     if isNull(d_ptr):
         d.putValue("(null)")
     else:
-        checkRef(d_ptr["ref"])
+        checkSimpleRef(d_ptr["ref"])
         d.putValue("(%dx%d)" % (d_ptr["width"], d_ptr["height"]))
     bits = d_ptr["data"]
     nbytes = d_ptr["nbytes"]
@@ -616,9 +634,7 @@ def qdumpHelper__QMap(d, value, forceLong):
 
         keyType = templateArgument(value.type, 0)
         valueType = templateArgument(value.type, 1)
-
-        isSimpleKey = isSimpleType(keyType)
-        isSimpleValue = isSimpleType(valueType)
+        isCompact = mapCompact(d.currentItemFormat(), keyType, valueType)
 
         it = e_ptr["forward"].dereference()
 
@@ -629,7 +645,7 @@ def qdumpHelper__QMap(d, value, forceLong):
         payloadSize = nodeType.sizeof - 2 * lookupType("void").pointer().sizeof
         charPtr = lookupType("char").pointer()
 
-        if isSimpleKey and isSimpleValue:
+        if isCompact:
             innerType = valueType
         else:
             innerType = nodeType
@@ -640,25 +656,27 @@ def qdumpHelper__QMap(d, value, forceLong):
                 base = it.cast(charPtr) - payloadSize
                 node = base.cast(nodeType.pointer()).dereference()
                 with SubItem(d, i):
-                    key = node["key"]
-                    val = node["value"]
-                    #if isSimpleType(value.type):
-                    # or isStringType(d, value.type):
-                    if isSimpleKey and isSimpleValue:
+                    if isCompact:
                         #d.putType(valueType)
                         if forceLong:
-                            d.putName("[%s] %s" % (i, key))
+                            d.putName("[%s] %s" % (i, node["key"]))
                         else:
-                            d.putName(key)
-                        d.putItem(val)
+                            d.putMapName(node["key"])
+                        d.putItem(node["value"])
                     else:
                         d.putItem(node)
                 it = it.dereference()["forward"].dereference()
 
 
+def qform__QMap():
+    return mapForms()
+
 def qdump__QMap(d, value):
     qdumpHelper__QMap(d, value, False)
 
+def qform__QMultiMap():
+    return mapForms()
+
 def qdump__QMultiMap(d, value):
     qdumpHelper__QMap(d, value, True)
 
@@ -1106,7 +1124,7 @@ def qdump__QPixmap(d, value):
     if isNull(d_ptr):
         d.putValue("(null)")
     else:
-        checkRef(d_ptr["ref"])
+        checkSimpleRef(d_ptr["ref"])
         d.putValue("(%dx%d)" % (d_ptr["w"], d_ptr["h"]))
     d.putNumChild(0)
 
@@ -1252,7 +1270,7 @@ def qdump__QSet(d, value):
         isSimpleKey = isSimpleType(keyType)
         node = hashDataFirstNode(value)
         innerType = e_ptr.dereference().type
-        with Children(d, size, numChild=1000, childType=keyType):
+        with Children(d, size, maxNumChild=1000, childType=innerType):
             for i in xrange(size):
                 it = node.dereference().cast(innerType)
                 with SubItem(d, i):
@@ -1626,23 +1644,29 @@ def qedit__QVector(expr, value):
 
 
 def qdump__QVector(d, value):
-    d_ptr = value["d"]
-    p_ptr = value["p"]
-    alloc = d_ptr["alloc"]
-    size = d_ptr["size"]
+    private = value["d"]
+    checkRef(private["ref"])
+    alloc = private["alloc"]
+    size = private["size"]
+    innerType = templateArgument(value.type, 0)
+    charPointerType = lookupType("char *")
+    try:
+        # Qt 5. Will fail on Qt 4 due to the missing 'offset' member.
+        offset = private["offset"]
+        data = private.cast(charPointerType) + offset
+    except:
+        # Qt 4.
+        data = value["p"]["array"]
+
+    p = data.cast(innerType.pointer())
 
     check(0 <= size and size <= alloc and alloc <= 1000 * 1000 * 1000)
-    checkRef(d_ptr["ref"])
-
-    innerType = templateArgument(value.type, 0)
     d.putItemCount(size)
     d.putNumChild(size)
     if d.isExpanded():
-        p = gdb.Value(p_ptr["array"]).cast(innerType.pointer())
-        charPtr = lookupType("char").pointer()
         d.putField("size", size)
         with Children(d, size, maxNumChild=2000, childType=innerType, addrBase=p,
-                addrStep=(p+1).cast(charPtr) - p.cast(charPtr)):
+                addrStep=(p+1).cast(charPointerType) - p.cast(charPointerType)):
             for i in d.childRange():
                 d.putSubItem(i, p.dereference())
                 p += 1
@@ -1682,12 +1706,39 @@ def qdump__QWeakPointer(d, value):
 def qdump__QxXmlAttributes(d, value):
     pass
 
+
 #######################################################################
 #
 # Standard Library dumper
 #
 #######################################################################
 
+def qdump__std__array(d, value):
+    size = numericTemplateArgument(value.type, 1)
+    d.putItemCount(size)
+    d.putNumChild(size)
+    if d.isExpanded():
+        innerType = templateArgument(value.type, 0)
+        with Children(d, size, childType=innerType):
+            pcur = value.address.cast(innerType.pointer())
+            for i in d.childRange():
+                d.putSubItem(i, pcur.dereference())
+                pcur += 1
+
+
+def qdump__std__complex(d, value):
+    innerType = templateArgument(value.type, 0)
+    base = value.address.cast(innerType.pointer())
+    real = base.dereference()
+    imag = (base + 1).dereference()
+    d.putValue("(%f, %f)" % (real, imag));
+    d.putNumChild(2)
+    if d.isExpanded():
+        with Children(d, 2, childType=innerType):
+            d.putSubItem("real", real)
+            d.putSubItem("imag", imag)
+
+
 def qdump__std__deque(d, value):
     innerType = templateArgument(value.type, 0)
     innerSize = innerType.sizeof
@@ -1745,6 +1796,9 @@ def qdump__std__list(d, value):
                 p = p["_M_next"]
 
 
+def qform__std__map():
+    return mapForms()
+
 def qdump__std__map(d, value):
     impl = value["_M_t"]["_M_impl"]
     size = impl["_M_node_count"]
@@ -1755,31 +1809,33 @@ def qdump__std__map(d, value):
     if d.isExpanded():
         keyType = templateArgument(value.type, 0)
         valueType = templateArgument(value.type, 1)
-        # Does not work on gcc 4.4, the allocator type (fourth template
-        # argument) seems not to be available.
-        #   pairType = templateArgument(templateArgument(value.type, 3), 0)
-        # So use this as workaround:
-        pairType = templateArgument(impl.type, 1)
-        isSimpleKey = isSimpleType(keyType)
-        isSimpleValue = isSimpleType(valueType)
+        try:
+            # Does not work on gcc 4.4, the allocator type (fourth template
+            # argument) seems not to be available.
+            pairType = templateArgument(templateArgument(value.type, 3), 0)
+            pairPointer = pairType.pointer()
+        except:
+            # So use this as workaround:
+            pairType = templateArgument(impl.type, 1)
+            pairPointer = pairType.pointer()
+        isCompact = mapCompact(d.currentItemFormat(), keyType, valueType)
         innerType = pairType
-        if isSimpleKey and isSimpleValue:
+        if isCompact:
             innerType = valueType
-        pairPointer = pairType.pointer()
         node = impl["_M_header"]["_M_left"]
         childType = innerType
         if size == 0:
             childType = pairType
         childNumChild = 2
-        if isSimpleKey and isSimpleValue:
+        if isCompact:
             childNumChild = None
         with Children(d, size, maxNumChild=1000,
                 childType=childType, childNumChild=childNumChild):
             for i in d.childRange():
                 with SubItem(d, i):
                     pair = (node + 1).cast(pairPointer).dereference()
-                    if isSimpleKey and isSimpleValue:
-                        d.putName(pair["first"])
+                    if isCompact:
+                        d.putMapName(pair["first"])
                         d.putItem(pair["second"])
                     else:
                         d.putValue(" ")
@@ -1885,6 +1941,45 @@ def qdump__std__string(d, value):
     d.putNumChild(0)
 
 
+def qdump__std__shared_ptr(d, value):
+    i = value["_M_ptr"]
+    if isNull(i):
+        d.putValue("(null)")
+        d.putNumChild(0)
+        return
+
+    if isSimpleType(templateArgument(value.type, 0)):
+        d.putValue("%s @0x%x" % (i.dereference(), long(i)))
+    else:
+        i = expensiveUpcast(i)
+        d.putValue("@0x%x" % long(i))
+
+    d.putNumChild(3)
+    with Children(d, 3):
+        d.putSubItem("data", i)
+        refcount = value["_M_refcount"]["_M_pi"]
+        d.putIntItem("usecount", refcount["_M_use_count"])
+        d.putIntItem("weakcount", refcount["_M_weak_count"])
+
+
+def qdump__std__unique_ptr(d, value):
+    i = value["_M_t"]["_M_head_impl"]
+    if isNull(i):
+        d.putValue("(null)")
+        d.putNumChild(0)
+        return
+
+    if isSimpleType(templateArgument(value.type, 0)):
+        d.putValue("%s @0x%x" % (i.dereference(), long(i)))
+    else:
+        i = expensiveUpcast(i)
+        d.putValue("@0x%x" % long(i))
+
+    d.putNumChild(1)
+    with Children(d, 1):
+        d.putSubItem("data", i)
+
+
 def qedit__std__vector(expr, value):
     values = value.split(',')
     n = len(values)
@@ -1905,8 +2000,9 @@ def qdump__std__vector(d, value):
     if isBool:
         start = impl["_M_start"]["_M_p"]
         finish = impl["_M_finish"]["_M_p"]
-        # FIXME: 32 is sizeof(unsigned long) * CHAR_BIT
-        storagesize = 32
+        # FIXME: 8 is CHAR_BIT
+        storage = lookupType("unsigned long")
+        storagesize = storage.sizeof * 8
         size = (finish - start) * storagesize
         size += impl["_M_finish"]["_M_offset"]
         size -= impl["_M_start"]["_M_offset"]
@@ -2040,6 +2136,22 @@ def qdump__boost__shared_ptr(d, value):
             d.putIntItem("usecount", usecount)
 
 
+def qdump__boost__gregorian__date(d, value):
+    d.putValue(value["days_"], JulianDate)
+    d.putNumChild(0)
+
+
+def qdump__boost__posix_time__ptime(d, item):
+    ms = long(item["time_"]["time_count_"]["value_"]) / 1000
+    d.putValue("%s/%s" % divmod(ms, 86400000), JulianDateAndMillisecondsSinceMidnight)
+    d.putNumChild(0)
+
+
+def qdump__boost__posix_time__time_duration(d, item):
+    d.putValue(long(item["ticks_"]["value_"]) / 1000, MillisecondsSinceMidnight)
+    d.putNumChild(0)
+
+
 #######################################################################
 #
 # Symbian
@@ -2302,6 +2414,50 @@ def qdump__Eigen__Matrix(d, value):
                         s = s + 1
 
 
+#######################################################################
+#
+# D
+#
+#######################################################################
+
+def cleanDType(type):
+    return stripClassTag(str(type)).replace("uns long long", "string")
+
+def qdump_Array(d, value):
+    n = value["length"]
+    p = value["ptr"]
+    t = cleanDType(value.type)[7:]
+    d.putAddress(value.address)
+    d.putType("%s[%d]" % (t, n))
+    if t == "char":
+        d.putValue(encodeCharArray(p, 100), Hex2EncodedLocal8Bit)
+        d.putNumChild(0)
+    else:
+        d.putValue(" ")
+        d.putNumChild(n)
+        innerType = p.type
+        if d.isExpanded():
+            with Children(d, n, childType=innerType):
+                for i in range(0, n):
+                    d.putSubItem(i, p.dereference())
+                    p = p + 1
+
+
+def qdump_AArray(d, value):
+    #n = value["length"]
+    # This ends up as _AArray_<key>_<value> with a single .ptr
+    # member of type void *. Not much that can be done here.
+    p = value["ptr"]
+    t = cleanDType(value.type)[8:]
+    d.putAddress(value.address)
+    d.putType("%s]" % t.replace("_", "["))
+    d.putValue(" ")
+    d.putNumChild(1)
+    if d.isExpanded():
+        with Children(d, 1):
+                d.putSubItem("ptr", p)
+
+
 #######################################################################
 #
 # Display Test
@@ -2438,3 +2594,26 @@ if False:
         d.putValue("This is the value: %s" % value["m_a"])
         d.putNoType()
         d.putNumChild(0)
+
+
+if False:
+    def qdump__bug6933__Base(d, value):
+        d.putValue("foo")
+        d.putPlainChildren(value)
+
+if False:
+    def qdump__gdb13393__Base(d, value):
+        d.putValue("Base (%s)" % value["a"])
+        d.putType(value.type)
+        d.putNumChild(1)
+        if d.isExpanded():
+            with Children(d):
+                d.putFields(value)
+
+    def qdump__gdb13393__Derived(d, value):
+        d.putValue("Derived (%s, %s)" % (value["a"], value["b"]))
+        d.putType(value.type)
+        d.putNumChild(1)
+        if d.isExpanded():
+            with Children(d):
+                d.putFields(value)
