@@ -2,7 +2,7 @@
 **
 ** This file is part of Qt Creator
 **
-** Copyright (c) 2011 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (c) 2012 Nokia Corporation and/or its subsidiary(-ies).
 **
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
@@ -46,11 +46,11 @@
 #include <botan/exceptn.h>
 #include <botan/init.h>
 
-#include <QtCore/QFile>
-#include <QtCore/QMutex>
-#include <QtCore/QMutexLocker>
-#include <QtNetwork/QNetworkProxy>
-#include <QtNetwork/QTcpSocket>
+#include <QFile>
+#include <QMutex>
+#include <QMutexLocker>
+#include <QNetworkProxy>
+#include <QTcpSocket>
 
 /*!
     \class Utils::SshConnection
@@ -76,6 +76,8 @@ namespace {
             Botan::LibraryInitializer::initialize("thread_safe=true");
             qRegisterMetaType<Utils::SshError>("Utils::SshError");
             qRegisterMetaType<Utils::SftpJobId>("Utils::SftpJobId");
+            qRegisterMetaType<Utils::SftpFileInfo>("Utils::SftpFileInfo");
+            qRegisterMetaType<QList <Utils::SftpFileInfo> >("QList<Utils::SftpFileInfo>");
             staticInitializationsDone = true;
         }
     }
@@ -165,9 +167,12 @@ SshConnectionParameters SshConnection::connectionParameters() const
     return d->m_connParams;
 }
 
-QAbstractSocket::NetworkLayerProtocol SshConnection::ipProtocolVersion() const
+SshConnectionInfo SshConnection::connectionInfo() const
 {
-    return d->m_socket->localAddress().protocol();
+    QTC_ASSERT(state() == Connected, return SshConnectionInfo());
+
+    return SshConnectionInfo(d->m_socket->localAddress(), d->m_socket->localPort(),
+        d->m_socket->peerAddress(), d->m_socket->peerPort());
 }
 
 SshConnection::~SshConnection()
@@ -321,12 +326,16 @@ void SshConnectionPrivate::handleIncomingData()
             e.errorString);
     } catch (Botan::Exception &e) {
         closeConnection(SSH_DISCONNECT_BY_APPLICATION, SshInternalError, "",
-            tr("Botan library exception: %1").arg(e.what()));
+            tr("Botan library exception: %1").arg(QString::fromAscii(e.what())));
     }
 }
 
 void SshConnectionPrivate::handleServerId()
 {
+#ifdef CREATOR_SSH_DEBUG
+    qDebug("%s: incoming data size = %d, incoming data = '%s'",
+        Q_FUNC_INFO, m_incomingData.count(), m_incomingData.data());
+#endif
     const int idOffset = m_incomingData.indexOf("SSH-");
     if (idOffset == -1)
         return;
@@ -453,12 +462,6 @@ void SshConnectionPrivate::handleServiceAcceptPacket()
         m_sendFacility.sendUserAuthByPwdRequestPacket(m_connParams.userName.toUtf8(),
             SshCapabilities::SshConnectionService, m_connParams.password.toUtf8());
     } else {
-        Utils::FileReader reader;
-        if (!reader.fetch(m_connParams.privateKeyFile))
-            throw SshClientException(SshKeyFileError,
-                tr("Private key error: %1").arg(reader.errorString()));
-
-        m_sendFacility.createAuthenticationKey(reader.data());
         m_sendFacility.sendUserAuthByKeyRequestPacket(m_connParams.userName.toUtf8(),
             SshCapabilities::SshConnectionService);
     }
@@ -637,6 +640,17 @@ void SshConnectionPrivate::connectToHost()
     m_error = SshNoError;
     m_ignoreNextPacket = false;
     m_errorString.clear();
+
+    try {
+        if (m_connParams.authenticationType == SshConnectionParameters::AuthenticationByKey)
+            createPrivateKey();
+    } catch (const SshClientException &ex) {
+        m_error = ex.error;
+        m_errorString = ex.errorString;
+        emit error(m_error);
+        return;
+    }
+
     connect(m_socket, SIGNAL(connected()), this, SLOT(handleSocketConnected()));
     connect(m_socket, SIGNAL(readyRead()), this, SLOT(handleIncomingData()));
     connect(m_socket, SIGNAL(error(QAbstractSocket::SocketError)), this,
@@ -681,7 +695,18 @@ void SshConnectionPrivate::closeConnection(SshErrorCode sshError,
 bool SshConnectionPrivate::canUseSocket() const
 {
     return m_socket->isValid()
-        && m_socket->state() == QAbstractSocket::ConnectedState;
+            && m_socket->state() == QAbstractSocket::ConnectedState;
+}
+
+void SshConnectionPrivate::createPrivateKey()
+{
+    Utils::FileReader reader;
+    if (m_connParams.privateKeyFile.isEmpty())
+        throw SshClientException(SshKeyFileError, tr("No private key file given."));
+    if (!reader.fetch(m_connParams.privateKeyFile))
+        throw SshClientException(SshKeyFileError,
+            tr("Private key file error: %1").arg(reader.errorString()));
+    m_sendFacility.createAuthenticationKey(reader.data());
 }
 
 QSharedPointer<SshRemoteProcess> SshConnectionPrivate::createRemoteProcess(const QByteArray &command)
