@@ -2,7 +2,7 @@
 **
 ** This file is part of Qt Creator
 **
-** Copyright (c) 2011 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (c) 2012 Nokia Corporation and/or its subsidiary(-ies).
 **
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
@@ -40,10 +40,10 @@
 
 #include <utils/savedaction.h>
 
-#include <QtCore/QDebug>
-#include <QtGui/QContextMenuEvent>
-#include <QtGui/QHeaderView>
-#include <QtGui/QMenu>
+#include <QDebug>
+#include <QContextMenuEvent>
+#include <QHeaderView>
+#include <QMenu>
 
 namespace Debugger {
 namespace Internal {
