@@ -2,37 +2,48 @@
 **
 ** This file is part of Qt Creator
 **
-** Copyright (c) 2009 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (c) 2012 Nokia Corporation and/or its subsidiary(-ies).
 **
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
-** Commercial Usage
-**
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Commercial License Agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Nokia.
 **
 ** GNU Lesser General Public License Usage
 **
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** This file may be used under the terms of the GNU Lesser General Public
+** License version 2.1 as published by the Free Software Foundation and
+** appearing in the file LICENSE.LGPL included in the packaging of this file.
+** Please review the following information to ensure the GNU Lesser General
+** Public License version 2.1 requirements will be met:
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** If you are unsure which license is appropriate for your use, please
-** contact the sales department at http://qt.nokia.com/contact.
+** In addition, as a special exception, Nokia gives you certain additional
+** rights. These rights are described in the Nokia Qt LGPL Exception
+** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+**
+** Other Usage
+**
+** Alternatively, this file may be used in accordance with the terms and
+** conditions contained in a signed written agreement between you and Nokia.
+**
+** If you have questions regarding the use of this file, please contact
+** Nokia at qt-info@nokia.com.
 **
 **************************************************************************/
 
 #include "formeditorplugin.h"
+#include "formwindoweditor.h"
+#include "formclasswizardpage.h"
 #include "qtcreatorintegration.h"
 #include "formeditorw.h"
-#include "formwindoweditor.h"
+#include "editordata.h"
+#include "codemodelhelpers.h"
+#include <widgethost.h>
 
-#include <cpptools/cppmodelmanagerinterface.h>
+#include <cpptools/cpprefactoringchanges.h>
+#include <cpptools/cpptoolsconstants.h>
+#include <cpptools/insertionpointlocator.h>
+#include <cpptools/symbolfinder.h>
+#include <cplusplus/ModelManagerInterface.h>
 #include <cplusplus/Symbols.h>
 #include <cplusplus/Overview.h>
 #include <cplusplus/CoreTypes.h>
@@ -41,19 +52,26 @@
 #include <cplusplus/Literals.h>
 #include <cplusplus/Scope.h>
 #include <cplusplus/Control.h>
-#include <cplusplus/LookupContext.h>
+#include <cplusplus/TranslationUnit.h>
 #include <coreplugin/icore.h>
+#include <coreplugin/mimedatabase.h>
 #include <coreplugin/editormanager/editormanager.h>
 #include <extensionsystem/pluginmanager.h>
 #include <texteditor/basetexteditor.h>
-#include <texteditor/itexteditable.h>
+#include <projectexplorer/projectexplorer.h>
+#include <projectexplorer/session.h>
+#include <utils/qtcassert.h>
+#include <utils/fileutils.h>
 
-#include <QtDesigner/QDesignerFormWindowInterface>
+#include <QDesignerFormWindowInterface>
+#include <QDesignerFormEditorInterface>
 
-#include <QtGui/QMessageBox>
+#include <QMessageBox>
 
-#include <QtCore/QFileInfo>
-#include <QtCore/QDebug>
+#include <QFileInfo>
+#include <QDir>
+#include <QDebug>
+#include <QUrl>
 
 enum { indentation = 4 };
 
@@ -65,43 +83,66 @@ static QString msgClassNotFound(const QString &uiClassName, const QList<Document
 {
     QString files;
     foreach (const Document::Ptr &doc, docList) {
-        if (!files.isEmpty())
-            files += QLatin1String(", ");
-        files += doc->fileName();
+        files += QLatin1Char('\n');
+        files += QDir::toNativeSeparators(doc->fileName());
     }
-    return QtCreatorIntegration::tr("The class definition of '%1' could not be found in %2.").arg(uiClassName, files);
-}
-
-static inline CppTools::CppModelManagerInterface *cppModelManagerInstance()
-{
-    return ExtensionSystem::PluginManager::instance()
-        ->getObject<CppTools::CppModelManagerInterface>();
+    return QtCreatorIntegration::tr(
+        "The class containing '%1' could not be found in %2.\n"
+        "Please verify the #include-directives.")
+        .arg(uiClassName, files);
 }
 
 QtCreatorIntegration::QtCreatorIntegration(QDesignerFormEditorInterface *core, FormEditorW *parent) :
-    qdesigner_internal::QDesignerIntegration(core, ::qobject_cast<QObject*>(parent)),
+#if QT_VERSION >= 0x050000
+    QDesignerIntegration(core, parent),
+#else
+    qdesigner_internal::QDesignerIntegration(core, parent),
+#endif
     m_few(parent)
 {
+#if QT_VERSION >= 0x050000
+    setResourceFileWatcherBehaviour(ReloadResourceFileSilently);
+    Feature f = features();
+    f |= SlotNavigationFeature;
+    f &= ~ResourceEditorFeature;
+    setFeatures(f);
+#else
     setResourceFileWatcherBehaviour(QDesignerIntegration::ReloadSilently);
     setResourceEditingEnabled(false);
     setSlotNavigationEnabled(true);
-    connect(this, SIGNAL(navigateToSlot(QString, QString, QStringList)),
-            this, SLOT(slotNavigateToSlot(QString, QString, QStringList)));
+#endif
+    connect(this, SIGNAL(navigateToSlot(QString,QString,QStringList)),
+            this, SLOT(slotNavigateToSlot(QString,QString,QStringList)));
+    connect(this, SIGNAL(helpRequested(QString,QString)),
+            this, SLOT(slotDesignerHelpRequested(QString,QString)));
+    slotSyncSettingsToDesigner();
+    connect(Core::ICore::instance(), SIGNAL(saveSettingsRequested()),
+            this, SLOT(slotSyncSettingsToDesigner()));
+}
+
+void QtCreatorIntegration::slotDesignerHelpRequested(const QString &manual, const QString &document)
+{
+    // Pass on as URL.
+    emit creatorHelpRequested(QUrl(QString::fromLatin1("qthelp://com.trolltech.%1/qdoc/%2")
+        .arg(manual, document)));
 }
 
 void QtCreatorIntegration::updateSelection()
 {
-    if (FormWindowEditor *afww = m_few->activeFormWindow())
-        afww->updateFormWindowSelectionHandles(true);
+    if (const EditorData ed = m_few->activeEditor())
+        ed.widgetHost->updateFormWindowSelectionHandles(true);
+#if QT_VERSION >= 0x050000
+    QDesignerIntegration::updateSelection();
+#else
     qdesigner_internal::QDesignerIntegration::updateSelection();
+#endif
 }
 
 QWidget *QtCreatorIntegration::containerWindow(QWidget * /*widget*/) const
 {
-    FormWindowEditor *fw = m_few->activeFormWindow();
-    if (!fw)
-        return 0;
-    return fw->integrationContainer();
+    if (const EditorData ed = m_few->activeEditor())
+        return ed.widgetHost->integrationContainer();
+    return 0;
 }
 
 static QList<Document::Ptr> findDocumentsIncluding(const CPlusPlus::Snapshot &docTable,
@@ -161,13 +202,13 @@ static const Class *findClass(const Namespace *parentNameSpace, const QString &c
 
     const Overview o;
     const unsigned namespaceMemberCount = parentNameSpace->memberCount();
-    for (unsigned i = 0; i < namespaceMemberCount; i++) { // we go through all namespace members
+    for (unsigned i = 0; i < namespaceMemberCount; ++i) { // we go through all namespace members
         const Symbol *sym = parentNameSpace->memberAt(i);
         // we have found a class - we are interested in classes only
         if (const Class *cl = sym->asClass()) {
             // 1) we go through class members
             const unsigned classMemberCount = cl->memberCount();
-            for (unsigned j = 0; j < classMemberCount; j++)
+            for (unsigned j = 0; j < classMemberCount; ++j)
                 if (const Declaration *decl = cl->memberAt(j)->asDeclaration()) {
                 // we want to know if the class contains a member (so we look into
                 // a declaration) of uiClassName type
@@ -198,16 +239,16 @@ static const Class *findClass(const Namespace *parentNameSpace, const QString &c
     return 0;
 }
 
-static const Function *findDeclaration(const Class *cl, const QString &functionName)
+static Function *findDeclaration(const Class *cl, const QString &functionName)
 {
     const QString funName = QString::fromUtf8(QMetaObject::normalizedSignature(functionName.toUtf8()));
     const unsigned mCount = cl->memberCount();
     // we are interested only in declarations (can be decl of method or of a field)
     // we are only interested in declarations of methods
     const Overview overview;
-    for (unsigned j = 0; j < mCount; j++) { // go through all members
-        if (const Declaration *decl = cl->memberAt(j)->asDeclaration())
-            if (const Function *fun = decl->type()->asFunctionType()) {
+    for (unsigned j = 0; j < mCount; ++j) { // go through all members
+        if (Declaration *decl = cl->memberAt(j)->asDeclaration())
+            if (Function *fun = decl->type()->asFunctionType()) {
                 // Format signature
                 QString memberFunction = overview.prettyName(fun->name());
                 memberFunction += QLatin1Char('(');
@@ -228,211 +269,65 @@ static const Function *findDeclaration(const Class *cl, const QString &functionN
     return 0;
 }
 
-// TODO: remove me, see below
-static bool isCompatible(const Name *name, const Name *otherName)
-{
-    if (const NameId *nameId = name->asNameId()) {
-        if (const TemplateNameId *otherTemplId = otherName->asTemplateNameId())
-            return nameId->identifier()->isEqualTo(otherTemplId->identifier());
-    } else if (const TemplateNameId *templId = name->asTemplateNameId()) {
-        if (const NameId *otherNameId = otherName->asNameId())
-            return templId->identifier()->isEqualTo(otherNameId->identifier());
-    }
-
-    return name->isEqualTo(otherName);
-}
-
-// TODO: remove me, see below
-static bool isCompatible(const Function *definition, const Symbol *declaration, const QualifiedNameId *declarationName)
-{
-    Function *declTy = declaration->type()->asFunctionType();
-    if (! declTy)
-        return false;
-
-    Name *definitionName = definition->name();
-    if (QualifiedNameId *q = definitionName->asQualifiedNameId()) {
-        if (! isCompatible(q->unqualifiedNameId(), declaration->name()))
-            return false;
-        else if (q->nameCount() > declarationName->nameCount())
-            return false;
-        else if (declTy->argumentCount() != definition->argumentCount())
-            return false;
-        else if (declTy->isConst() != definition->isConst())
-            return false;
-        else if (declTy->isVolatile() != definition->isVolatile())
-            return false;
-
-        for (unsigned i = 0; i < definition->argumentCount(); ++i) {
-            Symbol *arg = definition->argumentAt(i);
-            Symbol *otherArg = declTy->argumentAt(i);
-            if (! arg->type().isEqualTo(otherArg->type()))
-                return false;
-        }
-
-        for (unsigned i = 0; i != q->nameCount(); ++i) {
-            Name *n = q->nameAt(q->nameCount() - i - 1);
-            Name *m = declarationName->nameAt(declarationName->nameCount() - i - 1);
-            if (! isCompatible(n, m))
-                return false;
-        }
-        return true;
-    } else {
-        // ### TODO: implement isCompatible for unqualified name ids.
-    }
-    return false;
-}
-
 // TODO: remove me, this is taken from cppeditor.cpp. Find some common place for this method
-static Document::Ptr findDefinition(const Function *functionDeclaration, int *line)
+static Document::Ptr findDefinition(Function *functionDeclaration, int *line)
 {
-    CppTools::CppModelManagerInterface *cppModelManager = cppModelManagerInstance();
-    if (!cppModelManager)
-        return Document::Ptr();
+    if (CppModelManagerInterface *cppModelManager = CppModelManagerInterface::instance()) {
+        const Snapshot snapshot = cppModelManager->snapshot();
+        CppTools::SymbolFinder symbolFinder;
+        if (Symbol *def = symbolFinder.findMatchingDefinition(functionDeclaration, snapshot)) {
+            if (line)
+                *line = def->line();
 
-    QVector<Name *> qualifiedName;
-    Scope *scope = functionDeclaration->scope();
-    for (; scope; scope = scope->enclosingScope()) {
-        if (scope->isClassScope() || scope->isNamespaceScope()) {
-            if (scope->owner() && scope->owner()->name()) {
-                Name *scopeOwnerName = scope->owner()->name();
-                if (QualifiedNameId *q = scopeOwnerName->asQualifiedNameId()) {
-                    for (unsigned i = 0; i < q->nameCount(); ++i) {
-                        qualifiedName.prepend(q->nameAt(i));
-
-}
-                } else {
-                    qualifiedName.prepend(scopeOwnerName);
-                }
-            }
+            return snapshot.document(QString::fromUtf8(def->fileName(), def->fileNameLength()));
         }
     }
 
-    qualifiedName.append(functionDeclaration->name());
-
-    Control control;
-    QualifiedNameId *q = control.qualifiedNameId(&qualifiedName[0], qualifiedName.size());
-    LookupContext context(&control);
-    const Snapshot documents = cppModelManager->snapshot();
-    foreach (Document::Ptr doc, documents) {
-        QList<Scope *> visibleScopes;
-        visibleScopes.append(doc->globalSymbols());
-        visibleScopes = context.expand(visibleScopes);
-        foreach (Scope *visibleScope, visibleScopes) {
-            Symbol *symbol = 0;
-            if (NameId *nameId = q->unqualifiedNameId()->asNameId())
-                symbol = visibleScope->lookat(nameId->identifier());
-            else if (DestructorNameId *dtorId = q->unqualifiedNameId()->asDestructorNameId())
-                symbol = visibleScope->lookat(dtorId->identifier());
-            else if (TemplateNameId *templNameId = q->unqualifiedNameId()->asTemplateNameId())
-                symbol = visibleScope->lookat(templNameId->identifier());
-            else if (OperatorNameId *opId = q->unqualifiedNameId()->asOperatorNameId())
-                symbol = visibleScope->lookat(opId->kind());
-            // ### cast operators
-            for (; symbol; symbol = symbol->next()) {
-                if (! symbol->isFunction())
-                    continue;
-                else if (! isCompatible(symbol->asFunction(), functionDeclaration, q))
-                    continue;
-                *line = symbol->line(); // TODO: shift the line so that we are inside a function. Maybe just find the nearest '{'?
-                return doc;
-            }
-        }
-    }
     return Document::Ptr();
-
 }
 
-// TODO: Wait for robust Roberto's code using AST or whatever for that. Current implementation is hackish.
-static int findClassEndPosition(const QString &headerContents, int classStartPosition)
+static inline ITextEditor *editableAt(const QString &fileName, int line, int column)
 {
-    const QString contents = headerContents.mid(classStartPosition); // we start serching from the beginning of class declaration
-    // We need to find the position of class closing "}"
-    int openedBlocksCount = 0; // counter of nested {} blocks
-    int idx = 0; // index of current position in the contents
-    while (true) {
-        if (idx < 0 || idx >= contents.length()) // indexOf returned -1, that means we don't have closing comment mark
-            break;
-        if (contents.mid(idx, 2) == QLatin1String("//")) {
-            idx = contents.indexOf(QLatin1Char('\n'), idx + 2) + 1; // drop everything up to the end of line
-        } else if (contents.mid(idx, 2) == QLatin1String("/*")) {
-            idx = contents.indexOf(QLatin1String("*/"), idx + 2) + 1; // drop everything up to the nearest */
-        } else if (contents.mid(idx, 4) == QLatin1String("'\\\"'")) {
-            idx += 4; // drop it
-        } else if (contents.at(idx) == QLatin1Char('\"')) {
-            do {
-                idx = contents.indexOf(QLatin1Char('\"'), idx + 1); // drop everything up to the nearest "
-            } while (idx > 0 && contents.at(idx - 1) == QLatin1Char('\\')); // if the nearest " is preceeded by \ we find next one
-            if (idx < 0)
-                break;
-            idx++;
-        } else {
-            if (contents.at(idx) == QLatin1Char('{')) {
-                openedBlocksCount++;
-            } else if (contents.at(idx) == QLatin1Char('}')) {
-                openedBlocksCount--;
-                if (openedBlocksCount == 0) {
-                    return classStartPosition + idx;
-                }
-            }
-            idx++;
-        }
-    }
-    return -1;
+    return qobject_cast<ITextEditor *>(TextEditor::BaseTextEditorWidget::openEditorAt(fileName, line, column));
 }
 
-static inline ITextEditable *editableAt(const QString &fileName, int line, int column)
-{
-    return qobject_cast<ITextEditable *>(TextEditor::BaseTextEditor::openEditorAt(fileName, line, column));
-}
-
-static void addDeclaration(const QString &docFileName, const Class *cl, const QString &functionName)
+static void addDeclaration(const Snapshot &snapshot,
+                           const QString &fileName,
+                           const Class *cl,
+                           const QString &functionName)
 {
     QString declaration = QLatin1String("void ");
     declaration += functionName;
     declaration += QLatin1String(";\n");
 
-    // functionName comes already with argument names (if designer managed to
-    // do that).  First, let's try to find any method which is a private slot
-    // (then we don't need to add "private slots:" statement)
-    const unsigned mCount = cl->memberCount();
-    for (unsigned j = 0; j < mCount; j++) { // go through all members
-        if (const Declaration *decl = cl->memberAt(j)->asDeclaration())
-            if (const Function *fun = decl->type()->asFunctionType())  {
-                // we are only interested in declarations of methods.
-                if (fun->isSlot() && fun->isPrivate()) {
-                    const int line = fun->line(); // this is the beginning of function name: "void ^foo(...)"
-                    const int column = fun->column();
-                    if (ITextEditable *editable = editableAt(docFileName, line, column)) {
-                        unsigned dl, dc; // this position is the beginning of return value: "^void foo(...)"
-                        decl->getStartPosition(&dl, &dc);
-                        dc--; // if the first character in line is 'v' comming from "void" getStartPosition returns 1, not 0, so we always decrement it.
-                        editable->gotoLine(dl, dc);
-                        editable->position(ITextEditor::StartOfLine);
-                        const QString indentation = QString(dc, QLatin1Char(' '));
-                        editable->insert(declaration + indentation);
-                    }
-                    return;
-                }
-            }
-    }
+    CppTools::CppRefactoringChanges refactoring(snapshot);
+    CppTools::InsertionPointLocator find(refactoring);
+    const CppTools::InsertionLocation loc = find.methodDeclarationInClass(
+                fileName, cl, CppTools::InsertionPointLocator::PrivateSlot);
 
-    // We didn't find any method under "private slots:", let's add "private slots:". Below code
-    // adds "private slots:" by the end of the class definition.
+    //
+    //! \todo change this to use the Refactoring changes.
+    //
 
-    if (ITextEditable *editable = editableAt(docFileName, cl->line(), cl->column())) {
-        int classEndPosition = findClassEndPosition(editable->contents(), editable->position());
-        if (classEndPosition >= 0) {
-            int line, column;
-            editable->convertPosition(classEndPosition, &line, &column); // converts back position into a line and column
-            editable->gotoLine(line, column);  // go to position (we should be just before closing } of the class)
-            editable->insert(QLatin1String("\nprivate slots:\n    ") + declaration);
+    if (ITextEditor *editable = editableAt(fileName, loc.line(), loc.column() - 1)) {
+        BaseTextEditorWidget *editor = qobject_cast<BaseTextEditorWidget *>(editable->widget());
+        if (editor) {
+            QTextCursor tc = editor->textCursor();
+            int pos = tc.position();
+            tc.beginEditBlock();
+            tc.insertText(loc.prefix() + declaration + loc.suffix());
+            tc.setPosition(pos, QTextCursor::KeepAnchor);
+            editor->indentInsertedText(tc);
+            tc.endEditBlock();
         }
     }
 }
 
 static Document::Ptr addDefinition(const CPlusPlus::Snapshot &docTable,
-                                   const QString &headerFileName, const QString &className,
-                                   const QString &functionName, int *line)
+                                   const QString &headerFileName,
+                                   const QString &className,
+                                   const QString &functionName,
+                                   int *line)
 {
     QString definition = QLatin1String("\nvoid ");
     definition += className;
@@ -449,12 +344,20 @@ static Document::Ptr addDefinition(const CPlusPlus::Snapshot &docTable,
 
     QFileInfo headerFI(headerFileName);
     const QString headerBaseName = headerFI.completeBaseName();
-    const QString headerAbsolutePath = headerFI.absolutePath();
     foreach (const Document::Ptr &doc, docList) {
         const QFileInfo sourceFI(doc->fileName());
-        // we take only those documents which has the same filename and path (maybe we don't need to compare the path???)
-        if (headerBaseName == sourceFI.baseName() && headerAbsolutePath == sourceFI.absolutePath()) {
-            if (ITextEditable *editable = editableAt(doc->fileName(), 0, 0)) {
+        // we take only those documents which have the same filename
+        if (headerBaseName == sourceFI.baseName()) {
+            //
+            //! \todo change this to use the Refactoring changes.
+            //
+
+            if (ITextEditor *editable = editableAt(doc->fileName(), 0, 0)) {
+
+                //
+                //! \todo use the InsertionPointLocator to insert at the correct place.
+                // (we'll have to extend that class first to do definition insertions)
+
                 const QString contents = editable->contents();
                 int column;
                 editable->convertPosition(contents.length(), line, &column);
@@ -468,6 +371,51 @@ static Document::Ptr addDefinition(const CPlusPlus::Snapshot &docTable,
     return Document::Ptr();
 }
 
+static QString addConstRefIfNeeded(const QString &argument)
+{
+    if (argument.startsWith(QLatin1String("const "))
+            || argument.endsWith(QLatin1Char('&'))
+            || argument.endsWith(QLatin1Char('*')))
+        return argument;
+
+    // for those types we don't want to add "const &"
+    static const QStringList nonConstRefs = QStringList()
+            << QLatin1String("bool")
+            << QLatin1String("int")
+            << QLatin1String("uint")
+            << QLatin1String("float")
+            << QLatin1String("double")
+            << QLatin1String("long")
+            << QLatin1String("short")
+            << QLatin1String("char")
+            << QLatin1String("signed")
+            << QLatin1String("unsigned")
+            << QLatin1String("qint64")
+            << QLatin1String("quint64");
+
+    for (int i = 0; i < nonConstRefs.count(); i++) {
+        const QString nonConstRef = nonConstRefs.at(i);
+        if (argument == nonConstRef || argument.startsWith(nonConstRef + QLatin1Char(' ')))
+            return argument;
+    }
+    return QLatin1String("const ") + argument + QLatin1Char('&');
+}
+
+static QString formatArgument(const QString &argument)
+{
+    QString formattedArgument = argument;
+    int i = argument.count();
+    while (i > 0) { // from the end of the "argument" string
+        i--;
+        const QChar c = argument.at(i); // take the char
+        if (c != QLatin1Char('*') && c != QLatin1Char('&')) { // if it's not the * or &
+            formattedArgument.insert(i + 1, QLatin1Char(' ')); // insert space after that char or just append space (to separate it from the parameter name)
+            break;
+        }
+    }
+    return formattedArgument;
+}
+
 // Insert the parameter names into a signature, "void foo(bool)" ->
 // "void foo(bool checked)"
 static QString addParameterNames(const QString &functionSignature, const QStringList &parameterNames)
@@ -484,10 +432,19 @@ static QString addParameterNames(const QString &functionSignature, const QString
     for (int i = 0; i < aCount; ++i) {
         if (i > 0)
             functionName += QLatin1String(", ");
-        functionName += arguments.at(i);
+        const QString argument = addConstRefIfNeeded(arguments.at(i));
+        functionName += formatArgument(argument);
         if (i < pCount) {
-            functionName += QLatin1Char(' ');
-            functionName += parameterNames.at(i);
+            // prepare parameterName
+            QString parameterName = parameterNames.at(i);
+            if (parameterName.isEmpty()) {
+                const QString generatedName = QLatin1String("arg") + QString::number(i + 1);
+                if (!parameterNames.contains(generatedName))
+                    parameterName = generatedName;
+            }
+            // add parameterName if not empty
+            if (!parameterName.isEmpty())
+                functionName += parameterName;
         }
     }
     functionName += QLatin1Char(')');
@@ -514,8 +471,8 @@ static ClassDocumentPtrPair
         // Check the includes
         const unsigned recursionMaxIncludeDepth = maxIncludeDepth - 1u;
         foreach (const QString &include, doc->includedFiles()) {
-            const CPlusPlus::Snapshot::const_iterator it = docTable.constFind(include);
-            if (it != docTable.constEnd()) {
+            const CPlusPlus::Snapshot::const_iterator it = docTable.find(include);
+            if (it != docTable.end()) {
                 const Document::Ptr includeDoc = it.value();
                 const ClassDocumentPtrPair irc = findClassRecursively(docTable, it.value(), className, recursionMaxIncludeDepth, namespaceName);
                 if (irc.first)
@@ -546,6 +503,25 @@ static inline QString uiClassName(QString formObjectName)
     return formObjectName;
 }
 
+static Document::Ptr getParsedDocument(const QString &fileName, CppModelManagerInterface::WorkingCopy &workingCopy, Snapshot &snapshot)
+{
+    QString src;
+    if (workingCopy.contains(fileName)) {
+        src = workingCopy.source(fileName);
+    } else {
+        Utils::FileReader reader;
+        if (reader.fetch(fileName)) // ### FIXME error reporting
+            src = QString::fromLocal8Bit(reader.data()); // ### FIXME encoding
+    }
+
+    QByteArray source = snapshot.preprocessedCode(src, fileName);
+
+    Document::Ptr doc = snapshot.documentFromSource(source, fileName);
+    doc->check();
+    snapshot.insert(doc);
+    return doc;
+}
+
 // Goto slot invoked by the designer context menu. Either navigates
 // to an existing slot function or create a new one.
 
@@ -554,29 +530,58 @@ bool QtCreatorIntegration::navigateToSlot(const QString &objectName,
                                           const QStringList &parameterNames,
                                           QString *errorMessage)
 {
-    const QString currentUiFile = m_few->activeFormWindow()->file()->fileName();
+    typedef QMap<int, Document::Ptr> DocumentMap;
 
+    const EditorData ed = m_few->activeEditor();
+    QTC_ASSERT(ed, return false)
+    const QString currentUiFile = ed.formWindowEditor->document()->fileName();
+#if 0
+    return Designer::Internal::navigateToSlot(currentUiFile, objectName, signalSignature, parameterNames, errorMessage);
+#endif
     // TODO: we should pass to findDocumentsIncluding an absolute path to generated .h file from ui.
     // Currently we are guessing the name of ui_<>.h file and pass the file name only to the findDocumentsIncluding().
     // The idea is that the .pro file knows if the .ui files is inside, and the .pro file knows it will
     // be generating the ui_<>.h file for it, and the .pro file knows what the generated file's name and its absolute path will be.
     // So we should somehow get that info from project manager (?)
     const QFileInfo fi(currentUiFile);
+    const QString uiFolder = fi.absolutePath();
     const QString uicedName = QLatin1String("ui_") + fi.completeBaseName() + QLatin1String(".h");
 
-    // take all docs, find the ones that include the ui_xx.h.
+    // Retrieve code model snapshot restricted to project of ui file.
+    const ProjectExplorer::Project *uiProject = ProjectExplorer::ProjectExplorerPlugin::instance()->session()->projectForFile(currentUiFile);
+    if (!uiProject) {
+        *errorMessage = tr("Internal error: No project could be found for %1.").arg(currentUiFile);
+        return false;
+    }
+    CPlusPlus::Snapshot docTable = CppModelManagerInterface::instance()->snapshot();
+    CPlusPlus::Snapshot newDocTable;
 
-    const CPlusPlus::Snapshot docTable = cppModelManagerInstance()->snapshot();
-    QList<Document::Ptr> docList = findDocumentsIncluding(docTable, uicedName, true); // change to false when we know the absolute path to generated ui_<>.h file
+    for  (CPlusPlus::Snapshot::iterator it = docTable.begin(); it != docTable.end(); ++it) {
+        const ProjectExplorer::Project *project = ProjectExplorer::ProjectExplorerPlugin::instance()->session()->projectForFile(it.key());
+        if (project == uiProject)
+            newDocTable.insert(it.value());
+    }
+
+    docTable = newDocTable;
+
+    // take all docs, find the ones that include the ui_xx.h.
+    // Sort into a map, putting the ones whose path closely matches the ui-folder path
+    // first in case there are project subdirectories that contain identical file names.
+    const QList<Document::Ptr> docList = findDocumentsIncluding(docTable, uicedName, true); // change to false when we know the absolute path to generated ui_<>.h file
+    DocumentMap docMap;
+    foreach (const Document::Ptr &d, docList) {
+        const QFileInfo docFi(d->fileName());
+        docMap.insert(qAbs(docFi.absolutePath().compare(uiFolder, Qt::CaseInsensitive)), d);
+    }
 
     if (Designer::Constants::Internal::debug)
         qDebug() << Q_FUNC_INFO << objectName << signalSignature << "Looking for " << uicedName << " returned " << docList.size();
-    if (docList.isEmpty()) {
+    if (docMap.isEmpty()) {
         *errorMessage = tr("No documents matching '%1' could be found.\nRebuilding the project might help.").arg(uicedName);
         return false;
     }
 
-    QDesignerFormWindowInterface *fwi = m_few->activeFormWindow()->formWindow();
+    QDesignerFormWindowInterface *fwi = ed.widgetHost->formWindow();
 
     const QString uiClass = uiClassName(fwi->mainContainer()->objectName());
 
@@ -589,7 +594,7 @@ bool QtCreatorIntegration::navigateToSlot(const QString &objectName,
     const Class *cl = 0;
     Document::Ptr doc;
 
-    foreach (const Document::Ptr &d, docList) {
+    foreach (const Document::Ptr &d, docMap) {
         const ClassDocumentPtrPair cd = findClassRecursively(docTable, d, uiClass, 1u , &namespaceName);
         if (cd.first) {
             cl = cd.first;
@@ -616,7 +621,7 @@ bool QtCreatorIntegration::navigateToSlot(const QString &objectName,
     int line = 0;
     Document::Ptr sourceDoc;
 
-    if (const Function *fun = findDeclaration(cl, functionName)) {
+    if (Function *fun = findDeclaration(cl, functionName)) {
         sourceDoc = findDefinition(fun, &line);
         if (!sourceDoc) {
             // add function definition to cpp file
@@ -624,10 +629,14 @@ bool QtCreatorIntegration::navigateToSlot(const QString &objectName,
         }
     } else {
         // add function declaration to cl
-        addDeclaration(doc->fileName(), cl, functionNameWithParameterNames);
+        CppModelManagerInterface::WorkingCopy workingCopy =
+            CppModelManagerInterface::instance()->workingCopy();
+        const QString fileName = doc->fileName();
+        getParsedDocument(fileName, workingCopy, docTable);
+        addDeclaration(docTable, fileName, cl, functionNameWithParameterNames);
 
         // add function definition to cpp file
-        sourceDoc = addDefinition(docTable, doc->fileName(), className, functionNameWithParameterNames, &line);
+        sourceDoc = addDefinition(docTable, fileName, className, functionNameWithParameterNames, &line);
     }
 
     if (!sourceDoc) {
@@ -636,7 +645,17 @@ bool QtCreatorIntegration::navigateToSlot(const QString &objectName,
     }
 
     // jump to function definition, position within code
-    TextEditor::BaseTextEditor::openEditorAt(sourceDoc->fileName(), line + 2, indentation);
+    TextEditor::BaseTextEditorWidget::openEditorAt(sourceDoc->fileName(), line + 2, indentation);
 
     return true;
 }
+
+void QtCreatorIntegration::slotSyncSettingsToDesigner()
+{
+#if QT_VERSION > 0x040800
+    // Set promotion-relevant parameters on integration.
+    const Core::MimeDatabase *mdb = Core::ICore::mimeDatabase();
+    setHeaderSuffix(mdb->preferredSuffixByType(QLatin1String(CppTools::Constants::CPP_HEADER_MIMETYPE)));
+    setHeaderLowercase(FormClassWizardPage::lowercaseHeaderFiles());
+#endif
+}
