@@ -93,6 +93,7 @@ SessionManager::SessionManager(QObject *parent)
     m_sessionNode(new SessionNode(this)),
     m_sessionName(QLatin1String("default")),
     m_virginSession(true),
+    m_loadingSession(false),
     m_startupProject(0)
 {
     connect(ModeManager::instance(), SIGNAL(currentModeChanged(Core::IMode*)),
@@ -108,12 +109,6 @@ SessionManager::SessionManager(QObject *parent)
             this, SLOT(markSessionFileDirty()));
     connect(em, SIGNAL(editorsClosed(QList<Core::IEditor*>)),
             this, SLOT(markSessionFileDirty()));
-
-    m_autoSaveSessionTimer = new QTimer(this);
-    m_autoSaveSessionTimer->setSingleShot(true);
-    m_autoSaveSessionTimer->setInterval(10000);
-    connect(m_autoSaveSessionTimer, SIGNAL(timeout()),
-            ICore::instance(), SIGNAL(saveSettingsRequested()));
 }
 
 SessionManager::~SessionManager()
@@ -297,6 +292,11 @@ void SessionManager::removeProject(Project *project)
     removeProjects(QList<Project*>() << project);
 }
 
+bool SessionManager::loadingSession()
+{
+    return m_loadingSession;
+}
+
 bool SessionManager::save()
 {
     if (debug)
@@ -816,17 +816,23 @@ bool SessionManager::loadSession(const QString &session)
         }
     }
 
+    m_loadingSession = true;
+
     // Allow everyone to set something in the session and before saving
     emit aboutToUnloadSession(m_sessionName);
 
     if (!isDefaultVirgin()) {
-        if (!save())
+        if (!save()) {
+            m_loadingSession = false;
             return false;
+        }
     }
 
     // Clean up
-    if (!ICore::editorManager()->closeAllEditors())
+    if (!ICore::editorManager()->closeAllEditors()) {
+        m_loadingSession = false;
         return false;
+    }
 
     setStartupProject(0);
     removeProjects(projects());
@@ -878,6 +884,7 @@ bool SessionManager::loadSession(const QString &session)
 
     // Starts a event loop, better do that at the very end
     askUserAboutFailedProjects();
+    m_loadingSession = false;
     return true;
 }
 
@@ -899,7 +906,6 @@ void SessionManager::reportProjectLoadingProgress()
 
 void SessionManager::markSessionFileDirty(bool makeDefaultVirginDirty)
 {
-    m_autoSaveSessionTimer->start();
     if (makeDefaultVirginDirty)
         m_virginSession = false;
 }
