@@ -2,7 +2,7 @@
 **
 ** This file is part of Qt Creator
 **
-** Copyright (c) 2011 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (c) 2012 Nokia Corporation and/or its subsidiary(-ies).
 **
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
@@ -38,12 +38,12 @@
 #include "highlightermock.h"
 #include "formats.h"
 
-#include <QtCore/QSharedPointer>
-#include <QtCore/QScopedPointer>
-#include <QtCore/QList>
-#include <QtCore/QMetaType>
-#include <QtGui/QPlainTextEdit>
-#include <QtTest/QtTest>
+#include <QSharedPointer>
+#include <QScopedPointer>
+#include <QList>
+#include <QMetaType>
+#include <QPlainTextEdit>
+#include <QtTest>
 
 using namespace TextEditor;
 using namespace Internal;
